@@ -20,35 +20,45 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+from __future__ import absolute_import, division, print_function, \
+    with_statement
 
 import hashlib
 
 
 __all__ = ['ciphers']
 
-m2_not_found = False
-
 
 def create_cipher(alg, key, iv, op, key_as_bytes=0, d=None, salt=None,
                   i=1, padding=1):
-    global m2_not_found
-
     md5 = hashlib.md5()
     md5.update(key)
     md5.update(iv)
     rc4_key = md5.digest()
 
-    if not m2_not_found:
-        try:
-            import M2Crypto.EVP
-            return M2Crypto.EVP.Cipher('rc4', rc4_key, '', op, key_as_bytes=0,
-                                       d='md5', salt=None, i=1, padding=1)
-        except:
-            m2_not_found = True
-    import ctypes_openssl
-    return ctypes_openssl.CtypesCrypto('rc4', rc4_key, '', op)
+    try:
+        from shadowsocks.crypto import ctypes_openssl
+        return ctypes_openssl.CtypesCrypto(b'rc4', rc4_key, b'', op)
+    except:
+        import M2Crypto.EVP
+        return M2Crypto.EVP.Cipher(b'rc4', rc4_key, b'', op,
+                                   key_as_bytes=0, d='md5', salt=None, i=1,
+                                   padding=1)
 
 
 ciphers = {
-    'rc4-md5': (16, 16, create_cipher),
+    b'rc4-md5': (16, 16, create_cipher),
 }
+
+
+def test():
+    from shadowsocks.crypto import util
+
+    cipher = create_cipher(b'rc4-md5', b'k' * 32, b'i' * 16, 1)
+    decipher = create_cipher(b'rc4-md5', b'k' * 32, b'i' * 16, 1)
+
+    util.run_cipher(cipher, decipher)
+
+
+if __name__ == '__main__':
+    test()
