@@ -20,25 +20,32 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+from __future__ import absolute_import, division, print_function, \
+    with_statement
+
 import os
 import sys
 import hashlib
 import string
 import struct
 import logging
-import crypto.m2
-import crypto.rc4_md5
-import crypto.salsa20_ctr
-import crypto.ctypes_openssl
+
+from shadowsocks.crypto import m2, rc4_md5, salsa20_ctr, ctypes_openssl
 
 
 method_supported = {}
-
-method_supported.update(crypto.rc4_md5.ciphers)
-method_supported.update(crypto.salsa20_ctr.ciphers)
-method_supported.update(crypto.ctypes_openssl.ciphers)
+method_supported.update(rc4_md5.ciphers)
+method_supported.update(salsa20_ctr.ciphers)
+method_supported.update(ctypes_openssl.ciphers)
 # let M2Crypto override ctypes_openssl
-method_supported.update(crypto.m2.ciphers)
+method_supported.update(m2.ciphers)
+
+if hasattr(string, 'maketrans'):
+    maketrans = string.maketrans
+    translate = string.translate
+else:
+    maketrans = bytes.maketrans
+    translate = bytes.translate
 
 
 def random_string(length):
@@ -46,7 +53,6 @@ def random_string(length):
         import M2Crypto.Rand
         return M2Crypto.Rand.rand_bytes(length)
     except ImportError:
-        # TODO really strong enough on Linux?
         return os.urandom(length)
 
 
@@ -59,8 +65,8 @@ def get_table(key):
     m.update(key)
     s = m.digest()
     (a, b) = struct.unpack('<QQ', s)
-    table = [c for c in string.maketrans('', '')]
-    for i in xrange(1, 1024):
+    table = [c for c in maketrans(b'', b'')]
+    for i in range(1, 1024):
         table.sort(lambda x, y: int(a % (ord(x) + i) - a % (ord(y) + i)))
     return table
 
@@ -71,28 +77,24 @@ def init_table(key, method=None):
     if not method:
         if key in cached_tables:
             return cached_tables[key]
-        encrypt_table = ''.join(get_table(key))
-        decrypt_table = string.maketrans(encrypt_table,
-                                         string.maketrans('', ''))
+        encrypt_table = b''.join(get_table(key))
+        decrypt_table = maketrans(encrypt_table, maketrans(b'', b''))
         cached_tables[key] = [encrypt_table, decrypt_table]
     else:
-        try:
-            Encryptor(key, method)  # test if the settings if OK
-        except Exception as e:
-            logging.error(e)
-            sys.exit(1)
+        Encryptor(key, method)  # test if the settings if OK
 
 
 def EVP_BytesToKey(password, key_len, iv_len):
     # equivalent to OpenSSL's EVP_BytesToKey() with count 1
     # so that we make the same key and iv as nodejs version
-    password = str(password)
+    if hasattr(password, 'encode'):
+        password = password.encode('utf-8')
     r = cached_keys.get(password, None)
     if r:
         return r
     m = []
     i = 0
-    while len(''.join(m)) < (key_len + iv_len):
+    while len(b''.join(m)) < (key_len + iv_len):
         md5 = hashlib.md5()
         data = password
         if i > 0:
@@ -100,7 +102,7 @@ def EVP_BytesToKey(password, key_len, iv_len):
         md5.update(data)
         m.append(md5.digest())
         i += 1
-    ms = ''.join(m)
+    ms = b''.join(m)
     key = ms[:key_len]
     iv = ms[key_len:key_len + iv_len]
     cached_keys[password] = (key, iv)
@@ -109,13 +111,15 @@ def EVP_BytesToKey(password, key_len, iv_len):
 
 class Encryptor(object):
     def __init__(self, key, method=None):
-        if method == 'table':
+        if method == b'table':
+            if bytes != str:
+                raise Exception('table is not supported on Python 3')
             method = None
         self.key = key
         self.method = method
         self.iv = None
         self.iv_sent = False
-        self.cipher_iv = ''
+        self.cipher_iv = b''
         self.decipher = None
         if method:
             self.cipher = self.get_cipher(key, method, 1, iv=random_string(32))
@@ -132,7 +136,8 @@ class Encryptor(object):
         return len(self.cipher_iv)
 
     def get_cipher(self, password, method, op, iv=None):
-        password = password.encode('utf-8')
+        if hasattr(password, 'encode'):
+            password = password.encode('utf-8')
         method = method.lower()
         m = self.get_cipher_param(method)
         if m:
@@ -152,7 +157,7 @@ class Encryptor(object):
         if len(buf) == 0:
             return buf
         if not self.method:
-            return string.translate(buf, self.encrypt_table)
+            return translate(buf, self.encrypt_table)
         else:
             if self.iv_sent:
                 return self.cipher.update(buf)
@@ -164,7 +169,7 @@ class Encryptor(object):
         if len(buf) == 0:
             return buf
         if not self.method:
-            return string.translate(buf, self.decrypt_table)
+            return translate(buf, self.decrypt_table)
         else:
             if self.decipher is None:
                 decipher_iv_len = self.get_cipher_param(self.method)[1]
@@ -178,14 +183,14 @@ class Encryptor(object):
 
 
 def encrypt_all(password, method, op, data):
-    if method is not None and method.lower() == 'table':
+    if method is not None and method.lower() == b'table':
         method = None
     if not method:
         [encrypt_table, decrypt_table] = init_table(password)
         if op:
-            return string.translate(data, encrypt_table)
+            return translate(data, encrypt_table)
         else:
-            return string.translate(data, decrypt_table)
+            return translate(data, decrypt_table)
     else:
         result = []
         method = method.lower()
@@ -199,4 +204,4 @@ def encrypt_all(password, method, op, data):
             data = data[iv_len:]
         cipher = m(method, key, iv, op)
         result.append(cipher.update(data))
-        return ''.join(result)
+        return b''.join(result)
